//By linkcd 
//http://linkcd.cnblogs.com/

using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Framework;
using ArgsUtility;

namespace ArgsTest
{
    [TestFixture]
    public class ArgsTest
    {
        [Test]
        public void TestSimpleBooleanPresent()
        {
            Args args = new Args("x?", new String[] { "-x", "True" });
            Assert.IsTrue((bool)args['x']);

            args = new Args("x?", new String[] { "-x", "False" });
            Assert.IsFalse((bool)args['x']);
        }

        [Test]
        public void TestSimpleIngeterPresent()
        {
            Args args = new Args("x#", new String[] { "-x", "33" });
            Assert.AreEqual(33, args['x']);
        }

        [Test]
        public void TestSimpleBooleanAndIntegerPresent()
        {
            Args args = new Args("x?,i#", new String[] { "-x", "True", "-i", "23" });
            Assert.IsTrue((bool)args['x']);
            Assert.AreEqual(23, args['i']);
        }

        [Test]
        public void TestSimpleDoublePresent()
        {
            Args args = new Args("x&", new string[] { "-x", "42.3" });
            Assert.AreEqual(42.3, (double)args['x']);
        }

        [Test]
        public void TestSimpleStringPresent()
        {
            Args args = new Args("x*", new string[] {"-x", "this is a string"});
            Assert.AreEqual("this is a string", (string)args['x']);
        }

        [Test]
        public void TestComplexArguments()
        {
            /* this schema includes 7 arguments:
             * Integer Arguments: i, j
             * Boolean Arguments: b
             * Double Arguments: d, x
             * String Arguments: m, n
             */
            string schema = "i#,b?,j#,d&,m*,n*,x&";

            //the argument values
            string[] values = new string[] {
                "-i", "32",
                "-d", "33.4",
                "-m", "string 1", 
                "-b", "false",
                "-j", "53",
                "-x", "11.2",
                "-n", "string 2"};

            Args args = new Args(schema, values);

            Assert.AreEqual(32, (int)args['i']);
            Assert.AreEqual(33.4, (double)args['d']);
            Assert.AreEqual("string 2", (string)args['n']);
            Assert.AreEqual("string 1", (string)args['m']);
            Assert.AreEqual(53, (int)args['j']);
            Assert.IsFalse((bool)args['b']);
            Assert.AreEqual(11.2, (double)args['x']);
        }

        [Test]
        public void TestInvalidDouble()
        {
            Args args = new Args("x&", new string[] { "-x", "Three Point Two" });
            try
            {
                Assert.AreEqual(0, (double)args['x']);
                Assert.Fail("should NOT run this");
            }
            catch (Exception ex)
            {
                Assert.AreEqual(@"'Three Point Two' is an invalid Double format.", ex.Message);
            }
        }

        [Test]
        public void TestInvalidInteger()
        {
            Args args = new Args("x#", new string[] { "-x", "Two" });
            try
            {
                Assert.AreEqual(2, (int)args['x']);
                Assert.Fail("should NOT run this");
            }
            catch (Exception ex)
            {
                Assert.AreEqual(@"'Two' is an invalid Integer format.", ex.Message);
            }
        }

        [Test]
        public void TestWithNoSchemaButWithOneArgument()
        {
            try
            {
                Args args = new Args("", new String[] { "-x" });
                Assert.Fail("should NOT run this");
            }
            catch (Exception ex)
            {
                Assert.AreEqual("Null Schema", ex.Message);
            }
        }

        [Test]
        public void TestInvalidArgumentFormat()
        {
            try
            {
                new Args("f~", new String[] { });
                Assert.Fail("Args constructor should have throws exception");
            }
            catch (Exception ex)
            {
                Assert.AreEqual("Unknow Argument Symbol:~.", ex.Message);
            }
        }

        [Test]
        public void TestMissingStringArgument()
        {
           Args args = new Args("x*", new String[] { "-x" });
           try
           {
               string value = (string)args['x'];
               Assert.Fail("Should NOT run this");
           }
           catch (Exception ex)
           {
               Assert.AreEqual("Can't find String parameter for x.", ex.Message);
           }
        }

        [Test]
        public void TestMissingIntegerArgument()
        {
            Args args = new Args("x#", new String[] { "-x" });
            try
            {
                string value = (string)args['x'];
                Assert.Fail("Should NOT run this");
            }
            catch (Exception ex)
            {
                Assert.AreEqual("Can't find Integer parameter for x.", ex.Message);
            }
        }

        [Test]
        public void TestBadSchema()
        {
            try
            {
                Args args = new Args("x", new String[] { "-xy" });
                Assert.Fail("Should NOT run this");
            }
            catch (Exception ex)
            {
                Assert.AreEqual("Bad Schema: x", ex.Message);
            }
        }

        [Test]
        public void TestSpacesInFormat()
        {
            Args args = new Args("x*, y#", new String[] { "-x", "s1", "-y", "2" });
            Assert.AreEqual("s1", (string)args['x']);
            Assert.AreEqual(2, (int)args['y']);
        }

        [Test]
        public void TestUnexpectedMessage()
        {
            Args args = new Args("x*", new String[] { "-x", "s1"});
            try
            {
                string value = (string)args['m'];
                Assert.Fail("Should NOT run this");
            }
            catch (Exception ex)
            {
                Assert.AreEqual("Argument m unexpected.", ex.Message);
            }
        }

        [Test]
        public void Test2ArgumentsButMissingOne()
        {
            Args args = new Args("x#, y?", new String[] { "-y", "True", "-x"});
            try
            {
                Assert.IsTrue((bool)args['y']);
                string value = (string)args['x'];
                Assert.Fail("Should NOT run this");
            }
            catch (Exception ex)
            {
                Assert.AreEqual("Can't find Integer parameter for x.", ex.Message);
            }
        }

    }
}
